package com.sd.business.service.apply.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.apply.dto.ApplyBuyDto;
import com.sd.business.entity.apply.dto.ApplyBuySelectDto;
import com.sd.business.entity.apply.po.ApplyBuy;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.entity.apply.vo.ApplyBuyBomVo;
import com.sd.business.entity.apply.vo.ApplyBuyVo;
import com.sd.business.mapper.apply.ApplyBuyMapper;
import com.sd.business.service.apply.ApplyBuyBomService;
import com.sd.business.service.apply.ApplyBuyService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * <p>
 * 申购单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class ApplyBuyServiceImpl extends ServiceImpl<ApplyBuyMapper, ApplyBuy> implements ApplyBuyService {

    @Autowired
    private ApplyBuyBomService applyBuyBomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<ApplyBuyVo> getPage(ApplyBuySelectDto dto) {

        IWrapper<ApplyBuy> wrapper = getWrapper();
        wrapper.orderByDesc("ab", ApplyBuy::getId);

        // 申购单号
        wrapper.like("ab", ApplyBuy::getCode, dto.getCode());
        // 状态
        wrapper.eq("ab", ApplyBuy::getFlowStatus, dto.getFlowStatus());
        // 申购开始时间
        wrapper.ge("ab", ApplyBuy::getApplyTime, dto.getBeginTime());
        // 申购结束时间
        wrapper.le("ab", ApplyBuy::getApplyTime, dto.getEndTime());

        Page<ApplyBuyVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ApplyBuyVo detail(Long id) {
        Assert.notNull(id, "申购单id不能为空");

        // 申购单
        ApplyBuy ApplyBuy = this.getById(id);
        Assert.notNull(ApplyBuy, "没有找到申购单");

        // 申购明细
        List<ApplyBuyBom> list = applyBuyBomService.list(q -> q.eq(ApplyBuyBom::getApplyBuyId, id));
        List<ApplyBuyBomVo> applyBuyBomVoList = BeanUtil.copyToList(list, ApplyBuyBomVo.class);

        // 赋值bom规格
        bomSpecService.attributeAssign(applyBuyBomVoList, ApplyBuyBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
            item.setBomSpecColour(bomSpec.getColour());
            item.setBomSpecLength(bomSpec.getLength());
            item.setBomSpecWidth(bomSpec.getWidth());
            item.setBomSpecHeight(bomSpec.getHeight());
        });

        // TODO 库存，在途数量，近30天消耗量，近30天入库量，安全库存 未赋值

        // 赋值返回值
        ApplyBuyVo result = BeanUtil.toBean(ApplyBuy, ApplyBuyVo.class);
        result.setApplyBuyBomVoList(applyBuyBomVoList);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(ApplyBuyDto dto) {

        Long applyBuyId = dto.getId();
        Assert.notNull(applyBuyId, "申购单id不能为空");

        List<ApplyBuyBom> applyBuyBomList = dto.getApplyBuyBomList();
        applyBuyBomList.forEach(item -> item.setApplyBuyId(applyBuyId));

        updateById(dto);
        applyBuyBomService.editLinked(applyBuyBomList, ApplyBuyBom::getApplyBuyId, applyBuyId);

    }

}
